package jlowplugin.ui;

import java.awt.*;
import java.io.*;
import java.net.URL;

import javax.swing.*;
import javax.swing.tree.*;

import cds.jlow.client.codec.action.*;
import cds.jlow.client.descriptor.*;
import cds.jlow.client.graph.*;
import cds.jlow.client.graph.action.*;
import cds.jlow.client.graph.event.*;
import cds.jlow.client.sample.graph.event.*;
import cds.jlow.client.sample.service.action.*;
import cds.jlow.client.sample.tree.*;
import cds.jlow.client.sample.tree.event.*;
import cds.jlow.client.view.action.*;
import cds.jlow.server.motor.WorkManager;
import cds.jlow.server.motor.WorkRegister;
import cds.jlow.server.net.service.ServiceRegisterer;
import cds.jlow.server.sample.service.Arithmetic;

import jlowplugin.server.service.Blink;
import jlowplugin.server.service.Contour;
import jlowplugin.server.service.Export;
import jlowplugin.server.service.GetAladin;
import jlowplugin.server.service.GetLocal;
import jlowplugin.server.service.GetNED;
import jlowplugin.server.service.GetSimbad;
import jlowplugin.server.service.GetVizier;
import jlowplugin.server.service.RGB;
import jlowplugin.server.service.Sextractor;
import jlowplugin.server.service.XMatch;
import jlowplugin.ui.action.*;
import jlowplugin.ui.tree.*;
import jlowplugin.ui.tree.event.*;
import jlowplugin.util.Resources;
import jlowplugin.*;
//import jlowplugin.server.*;

/**
 * 
 * @author Cyril Pestel
 * @version 0.1
 */
public class JlowFrame extends JFrame {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 8399570901649683995L;

	protected static Resources resources = new Resources();
	
	protected IRegisterer register; // ?? keep or not

	protected RegisterInstance instances;
	protected DefaultMenu menu;
	
	protected StateBar stateBar;
	protected ToolBar toolbar;
	
	protected Jlow wfEditor;
	
	protected ScriptPane scriptPane;
	
	protected DescriptorTree tree;
	
	private JSplitPane splitVpane;
	
	private JSplitPane splitHpane;
	
	public JlowFrame() {
		
		setSize(800, 600);
		setLocation(100, 100);
//		setTitle("Aladin workflow builder v. 0.4 - Using JLOW library");

		setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
		
		//Register of tasks
		register = new PriorityRegisterer();
		
		//Main class of JLOW
		wfEditor = new Jlow(register);

		// For Alt+Enter==Properties Panel
		wfEditor.addKeyListener(new SimpleKeyListener(this, wfEditor));
		// For DoubleClick==Properties Panel
//		wfEditor.addMouseListener(new SimpleMouseListener(wfEditor, new CellPropertyAction(wfEditor, new PropertyFrameAction(this))));
		wfEditor.addMouseListener(new SimpleMouseListener(wfEditor, new CellPropertyAction(wfEditor, new PropertyAction(this, wfEditor))));
		
		// Add data
		try {
			wfEditor.registerCell("cds.jlow.descriptor.IntegerDescriptor", Class.forName("cds.jlow.client.graph.cell.DataCell"));
			wfEditor.registerCell("cds.jlow.descriptor.StringDescriptor", Class.forName("cds.jlow.client.graph.cell.DataCell"));
		} catch (ClassNotFoundException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		
		//
//		wfEditor.addGraphDescriptorListener(new DefaultGDListener());
		wfEditor.addGraphDescriptorListener(new CustomGDListener());
		
		//ScrollBar
		JScrollPane scrollPaneGraph = new JScrollPane((Jlow) wfEditor);

		//WheelMouse (zoom)
		wfEditor.addMouseWheelListener(new DefaultMouseWheelListener(wfEditor, scrollPaneGraph));
		
		//Drag&Drop
		wfEditor.setDropTargetListener(new DefaultDTL(wfEditor));

		//Popup Menu
		DefaultMPopupMenu mpm = new DefaultMPopupMenu(wfEditor);
		initPopupMenu(mpm);
		wfEditor.setPopupMenus(mpm);

		// Force type cast
		((DefaultMarqueeHandler) wfEditor.getMarqueeHandler()).getCheckGraph().setCast(true); 
		
		// ** Command Panel **
		scriptPane = new ScriptPane();
		
//		ScrollBar
		JScrollPane scrollPaneCommand = new JScrollPane(scriptPane);
//		scrollPaneCommand.setMinimumSize(new Dimension(400, 200));
//		scrollPaneCommand.setPreferredSize(new Dimension(400, 100));
//		scrollPaneCommand.setMaximumSize(new Dimension(400, 100));
		
		splitHpane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, scrollPaneGraph,
				scrollPaneCommand);
		
		splitHpane.setContinuousLayout(true);
		splitHpane.setDividerSize(4);
		splitHpane.setResizeWeight(1);
		
		
		// ** List **
		
		AladinListService aladinList = new AladinListService();
		aladinList.load(register);
		
//		@@ Arbre graphique contenant les descripteurs
		tree = new AladinDescriptorTree(new DefaultMutableTreeNode("Tasks"));
		tree.addDragSourceListener(new DefaultDragSAdapter(tree));
		tree.setDragGestureListener(new DefaultDragGListener(tree));
		DefaultSelectionListener selectionListener = new DefaultSelectionListener(wfEditor);
//		tree.addTreeSelectionListener(selectionListener);
		selectionListener.addTree(tree);
		
		JScrollPane scrollPaneDesc = new JScrollPane(tree);

//		Load
		CustomTreeManager.loadTask(tree, register);
		
//		@@ Arbre graphique contenant les data
		DescriptorTree treeData = new AladinDescriptorTree(new DefaultMutableTreeNode("Data"));
		treeData.addDragSourceListener(new DefaultDragSAdapter(treeData));
		treeData.setDragGestureListener(new DefaultDragGListener(treeData));
//		treeData.addTreeSelectionListener(selectionListener);
		selectionListener.addTree(treeData);
		
		JScrollPane scrollPaneData = new JScrollPane(treeData);

//		Load
		CustomTreeManager.loadData(treeData, register);
		
		JSplitPane splitpaneTree = new JSplitPane(JSplitPane.VERTICAL_SPLIT, scrollPaneDesc,
				scrollPaneData);
		splitpaneTree.setContinuousLayout(false);
		splitpaneTree.setOneTouchExpandable(false);
		splitpaneTree.setDividerLocation(400);
		splitpaneTree.setDividerSize(1);
		
		splitVpane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, splitpaneTree,
				splitHpane);
		splitVpane.setContinuousLayout(true);
		splitVpane.setDividerLocation(0.2);
		splitVpane.setDividerSize(8);
		
		Container content = getContentPane();
		content.setLayout(new BorderLayout());
		
		content.add(splitVpane, "Center");
		

		// ** MENU ** \\
		instances = new RegisterInstance();
		menu = new DefaultMenu();
		
		initActionMenu();
		
		readMenuFile();
		
		setJMenuBar(menu);
		
		// ** TOOL BAR ** \\
		toolbar = new ToolBar(wfEditor);
		initToolBar();
		content.add(toolbar, BorderLayout.NORTH);
		
		// ** STATE BAR ** \\
		stateBar = new StateBar();
		content.add(stateBar, "South");
		
		tree.addMouseMotionListener(new TreeMouseMotionListener(tree, stateBar));

		CommentMouseListener commentListener = new CommentMouseListener(stateBar);
		
		wfEditor.addMouseListener(commentListener);
		scriptPane.addMouseListener(commentListener);
		scrollPaneCommand.addMouseListener(commentListener);
	}
	
	public void initPopupMenu(DefaultMPopupMenu mpm) {
		mpm.initPopup();

		// For the graph
		mpm.insertItem(IManagerPopupMenu.GRAPHMENU, 0, new LoadAsAction(wfEditor));
		mpm.insertItem(IManagerPopupMenu.GRAPHMENU, 1, new SaveAsAction(wfEditor));
		
//		mpm.addSeparator(IManagerPopupMenu.GRAPHMENU);
		
//		mpm.addItem(IManagerPopupMenu.GRAPHMENU, new ExecServAction(wfEditor));
		
		mpm.addSeparator(IManagerPopupMenu.GRAPHMENU);
		JMenu submenudata = mpm.addSubMenu(IManagerPopupMenu.GRAPHMENU, "Data");
		submenudata.add(new InsertAction(wfEditor, "com.data.string", "String"));
		
		mpm.addSeparator(IManagerPopupMenu.DATAMENU);
		mpm.addItem(IManagerPopupMenu.DATAMENU, new PropertyAction(this, wfEditor));

		// For the task
		mpm.addSeparator(IManagerPopupMenu.TASKMENU);
		mpm.addItem(IManagerPopupMenu.TASKMENU, new PropertyAction(this, wfEditor));
		
		// For the port
		mpm.addItem(IManagerPopupMenu.PORTMENU, new PropertyAction(this, wfEditor));
		
		
	}
	 
	public void initActionMenu() {
		// File
		instances.addInstance("actionhide", new ActionHide(this));
		instances.addInstance("actionopen", new LoadAsAction(wfEditor));
		instances.addInstance("actionsave", new SaveAsAction(wfEditor));
		instances.addInstance("actionimport", new ImportAsAction(wfEditor));
		
		// Edit
//		instances.addInstance("actionexecute", new ExecServAction(wfEditor));
		
		// View
		instances.addInstance("actionscript", new SwitchScriptPaneAction(this));
		instances.addInstance("actionsidebar", new SwitchSidebarAction(this));
		instances.addInstance("actionzoomin", new ZoomInAction(wfEditor));
		instances.addInstance("actionzoomout", new ZoomOutAction(wfEditor));
		
		ActionInserted aInserted = new ActionInserted(wfEditor);
		aInserted.putValue("Load Aladin", "get_aladin");
		//aInserted.putValue("Load Vizier", "get_vizier");
		//aInserted.putValue("Load Simbad", "get_simbad");
		
		instances.addInstance("ainserted", aInserted);
		
		// Help
		instances.addInstance("actionabout", new AboutAction(new AboutDialog(this)));
		instances.addInstance("actionhelp", new AboutAction(new HelpContentsDialog(this)));
	}
	/**
	 * Initializes the tool bar.<br>
	 * Depends on the method initActionMenu.
	 */
	protected void initToolBar() {
		toolbar.addToolBar(new JToolBar("filebar", JToolBar.HORIZONTAL));
//		URL imageURL = this.getClass().getResource("/images/folder.png");
		toolbar.addAction(0, (Action) instances.getInstance("actionopen"), "Load a workflow", "Load As...", resources.getImage("folder"));
//		imageURL = this.getClass().getResource("/images/disk.png");
		toolbar.addAction(0, (Action) instances.getInstance("actionsave"), "Save the current workflow", "Save As...", resources.getImage("disk"));
		
	}
	
	protected void readMenuFile() {
		InputStream input = null;
		File menufile = new File("./resources"+File.separator+"menu.xml");
		try {
			
			if(menufile.exists()) {
				input = new BufferedInputStream(new FileInputStream(menufile));
				System.out.println("menu.xml detected");
			}
			else
				input = this.getClass().getResourceAsStream("/menu.xml");
			
			//  Initialize
		
			//DefaultMenu.createMenu("menu.xml", menu, instances);
			if(input!=null)
				DefaultMenu.createMenu(input, menu, instances);
			
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public static void main(String argv[]) {
		JlowFrame frame = new JlowFrame();
		frame.setVisible(true);
		frame.getSplitHpane().setDividerLocation(0.8);
//		frame.hideScriptPane();
	}

	public Jlow getWfEditor() {
		return wfEditor;
	}

	
	/**
	 * @return the splitVpane
	 */
	public JSplitPane getSplitVpane() {
		return splitVpane;
	}

	/**
	 * @return the splitHpane
	 */
	public JSplitPane getSplitHpane() {
		return splitHpane;
	}

	/**
	 * @return the scriptPane
	 */
	public ScriptPane getScriptPane() {
		return scriptPane;
	}
	
}
