package jlowplugin.ui;


import org.kxml2.io.*;
import org.xmlpull.v1.*;

import java.io.*;

//import java.util.*;

import javax.swing.*;
import java.awt.event.*;

/**
 * Class for creating a menu with a xml file.
 * 
 * @author Cyril Pestel
 */
public class KMenu {

	public static String MENUBAR = "menubar";
	public static String MENU = "menu";
	public static String ITEM = "item";
	public static String SEPARATOR = "separator";
	public static String ACTION = "action";
	
	public static String NAME = "name";
	public static String MNEMONIC = "mne";
	public static String POSITION = "pos";
	public static String ACCELERATOR = "accel";
	public static String ACTIONLISTENER = "al";
	public static String OBJECT = "object";
	public static String METHOD = "method";
	
	public static RegisterInstance register;
	
	public static void decode(InputStream inputStream, AbstractMenu menubar, RegisterInstance reg) throws Exception {
		decode(inputStream, new KXmlParser(), menubar, reg);
	}
	
	public static void decode(InputStream inputStream, XmlPullParser parser, AbstractMenu menubar, RegisterInstance reg) throws Exception {
	 	parser.setInput(inputStream, "UTF-8");
	 	
	 	register = reg;
	 	decodeMenubar(parser, menubar);
	 	
	 }
	 
	/**
	 * Decode the tag <code>menubar</code>.
	 *
	 */
	 public static void decodeMenubar(XmlPullParser parser, AbstractMenu menubar) throws Exception {
	 	parser.nextTag ();
	 	
	 	parser.require(XmlPullParser.START_TAG, null, KMenu.MENUBAR);
	 	
//	 	Iterate through our XML file
		while(parser.nextTag () != XmlPullParser.END_TAG) {
			
			decodeMenu(parser, menubar);
		}
	 	
	 	parser.require(XmlPullParser.END_TAG, null, KMenu.MENUBAR);
	 	//parser.nextTag ();
	 }
	 /**
	  * 
	  * @param parser
	  * @param menu
	  * @throws Exception
	  */
	 public static void decodeMenu(XmlPullParser parser, AbstractMenu menubar) throws Exception {
	 	parser.require(XmlPullParser.START_TAG, null, KMenu.MENU);
	 	
	 	String name = parser.getAttributeValue(null, KMenu.NAME);
	 	String mne = parser.getAttributeValue(null, KMenu.MNEMONIC);
	 	
	 	JMenu menu = AbstractMenu.getMenu(menubar, name);
	 	//Tests if the menu exist
	 	if(menu==null)
	 		menu = menubar.addMenu(name, mne);
	 	
	 	//ArrayList litem = new ArrayList();
	 	
//	 	Iterate through our XML file
		while(parser.nextTag () != XmlPullParser.END_TAG) {
			
			//decodeItem(parser, menu)
			//JMenuItem item = 
			String tag = parser.getName();
			//System.out.println("tag : "+tag);
			
			if(tag.equals(KMenu.ITEM))
				decodeItem(parser, menu);
			else if(tag.equals(KMenu.SEPARATOR))
				decodeSepartor(parser, menu);
			//if(item!=null)
			//	litem.add(item);
		}
		
		//parser.nextTag ();
	 	parser.require(XmlPullParser.END_TAG, null, KMenu.MENU);
	 	//parser.nextTag ();
	 }
	 
	 public static void decodeItem(XmlPullParser parser, JMenu menu) throws Exception {
	 	parser.require(XmlPullParser.START_TAG, null, KMenu.ITEM);
	 	
	 	//JMenuItem item = null;
	 	
	 	String name = parser.getAttributeValue(null, KMenu.NAME);
	 	String pos = parser.getAttributeValue(null, KMenu.POSITION);
	 	String mne = parser.getAttributeValue(null, KMenu.MNEMONIC);
	 	String accel = parser.getAttributeValue(null, KMenu.ACCELERATOR);
	 	//String al = parser.getAttributeValue(null, KMenu.ACTIONLISTENER);
	 	
	 	ActionListener actionl = null;
	 	
	 	if(parser.nextTag () != XmlPullParser.END_TAG) {
	 		actionl = decodeAction(parser);
	 	}
	 	
	 	//Insert a the last position
	 	int positem = menu.getItemCount();
	 	if(pos!=null) {
	 		try {
	 			
	 			positem = Integer.parseInt(pos);
	 			
	 		}
	 		catch(NumberFormatException nfe) {
	 			System.err.println("KMenu.decodeItem : "+nfe);
	 			nfe.printStackTrace();
	 		}
	 	}
	 	
	 	if(name!=null) {
	 		
	 		
	 		
	 		if(AbstractMenu.getItem(menu, name)==null)
	 			AbstractMenu.insert(menu, positem, name, accel, mne, null, actionl, null);
	 	}
	 	
	 	parser.nextTag ();
	 	parser.require(XmlPullParser.END_TAG, null, KMenu.ITEM);
	 	//parser.nextTag ();
	 	//return item;
	 }
	 
	 public static void decodeSepartor(XmlPullParser parser, JMenu menu) throws Exception {
		 parser.require(XmlPullParser.START_TAG, null, KMenu.SEPARATOR);
		 
		 menu.addSeparator();
		 //System.out.println("Separator");
		 parser.nextTag ();
		 parser.require(XmlPullParser.END_TAG, null, KMenu.SEPARATOR);
		 
		 //parser.nextTag ();
	 }
	 
	 public static ActionListener decodeAction(XmlPullParser parser) throws Exception {
	 	parser.require(XmlPullParser.START_TAG, null, KMenu.ACTION);
	 	
	 	String name = parser.getAttributeValue(null, KMenu.NAME);
	 	String obj = parser.getAttributeValue(null, KMenu.OBJECT);
	 	String method = parser.getAttributeValue(null, KMenu.METHOD);
	 	
	 	ActionListener al = null;
	 	
	 	if(obj!=null) {
	 		if(name!=null && method!=null) {
	 			Object o = register.getInstance(obj);
	 			if(o!=null)
	 				al = new ActionRedirect(name, o, method);	// Load a new instance not register bu known
	 				
	 		}
	 		else {
	 			al = (ActionListener) register.getInstance(obj);	
	 			//System.out.println("action: "+al);	
	 		}
	 			
	 	}
	 		
	 	parser.nextTag ();
	 	parser.require(XmlPullParser.END_TAG, null, KMenu.ACTION);
	 	
	 	//parser.nextTag ();
	 	
	 	return al;
	 }
	
}
